@@ -4,7 +4,7 @@
  * Created Date: 24/05/2021
  * Author: Shun Suzuki
  * -----
- * Last Modified: 02/10/2021
+ * Last Modified: 14/10/2021
  * Modified By: Shun Suzuki (suzuki@hapis.k.u-tokyo.ac.jp)
  * -----
  * Copyright (c) 2021 Hapis Lab. All rights reserved.
@@ -18,22 +18,26 @@ use crate::{
     gain::Gain,
     geometry::Geometry,
     hardware_defined::{
-        CPUControlFlags, CommandType, FPGAControlFlags, GlobalHeader, SeqFocus, MOD_FRAME_SIZE,
+        CPUControlFlags, FPGAControlFlags, GlobalHeader, SeqFocus, MOD_FRAME_SIZE, MSG_NORMAL_BASE,
         NUM_TRANS_IN_UNIT,
     },
     modulation::Modulation,
     sequence::{GainSequence, PointSequence, Sequence},
 };
 
-static MSG_ID: AtomicU8 = AtomicU8::new(0);
+static MSG_ID: AtomicU8 = AtomicU8::new(MSG_NORMAL_BASE);
 
 pub struct Logic {}
 
 impl Logic {
-    fn get_id() -> u8 {
+    pub fn get_id() -> u8 {
         MSG_ID.fetch_add(1, atomic::Ordering::SeqCst);
-        let _ =
-            MSG_ID.compare_exchange(0xFF, 1, atomic::Ordering::SeqCst, atomic::Ordering::SeqCst);
+        let _ = MSG_ID.compare_exchange(
+            0xFF,
+            MSG_NORMAL_BASE,
+            atomic::Ordering::SeqCst,
+            atomic::Ordering::SeqCst,
+        );
         MSG_ID.load(atomic::Ordering::SeqCst)
     }
 
@@ -49,45 +53,46 @@ impl Logic {
     }
 
     pub fn pack_header(
-        cmd: CommandType,
-        flag: FPGAControlFlags,
-        cmd_flag: CPUControlFlags,
+        msg_id: u8,
+        fpga_flag: FPGAControlFlags,
+        cpu_flag: CPUControlFlags,
         data: &mut [u8],
-        msg_id: &mut u8,
     ) {
         let header = data.as_mut_ptr() as *mut GlobalHeader;
-        *msg_id = Self::get_id();
         unsafe {
-            (*header).msg_id = *msg_id;
-            (*header).ctrl_flag = flag;
-            (*header).command = cmd;
-            (*header).command_flag = cmd_flag;
+            (*header).msg_id = msg_id;
+            (*header).fpga_flag = fpga_flag;
+            (*header).cpu_flag = cpu_flag;
             (*header).mod_size = 0;
         }
     }
 
     pub fn pack_header_mod<M: Modulation>(
         modulation: &mut M,
-        flag: FPGAControlFlags,
-        cmd_flag: CPUControlFlags,
+        fpga_flag: FPGAControlFlags,
+        cpu_flag: CPUControlFlags,
         data: &mut [u8],
-        msg_id: &mut u8,
-    ) {
-        Self::pack_header(CommandType::Op, flag, cmd_flag, data, msg_id);
+    ) -> u8 {
+        let msg_id = Self::get_id();
+        Self::pack_header(msg_id, fpga_flag, cpu_flag, data);
+        if modulation.finished() {
+            return msg_id;
+        }
+
         unsafe {
             let header = data.as_mut_ptr() as *mut GlobalHeader;
             let mut offset = 0;
             if modulation.sent() == 0 {
-                (*header).command_flag |= CPUControlFlags::MOD_BEGIN;
-                (*header).mod_data[0] = (modulation.sampling_frequency_division() & 0xFF) as u8;
-                (*header).mod_data[1] =
-                    (modulation.sampling_frequency_division() >> 8 & 0xFF) as u8;
+                (*header).cpu_flag |= CPUControlFlags::MOD_BEGIN;
+                let div = (*modulation.sampling_frequency_division() - 1) as u16;
+                (*header).mod_data[0] = (div & 0xFF) as u8;
+                (*header).mod_data[1] = (div >> 8 & 0xFF) as u8;
                 offset += 2;
             }
             let mod_size = modulation.remaining().clamp(0, MOD_FRAME_SIZE - offset);
             (*header).mod_size = mod_size as u8;
             if modulation.sent() + mod_size >= modulation.buffer().len() {
-                (*header).command_flag |= CPUControlFlags::MOD_END;
+                (*header).cpu_flag |= CPUControlFlags::MOD_END;
             }
             std::ptr::copy_nonoverlapping(
                 modulation.head(),
@@ -96,52 +101,50 @@ impl Logic {
             );
             modulation.send(mod_size);
         }
+
+        msg_id
     }
 
-    pub fn pack_body<G: Gain>(gain: &G, data: &mut [u8], size: &mut usize) {
+    pub fn pack_body<G: Gain>(gain: &G, data: &mut [u8]) -> usize {
         let num_devices = gain.data().len();
 
-        *size = std::mem::size_of::<GlobalHeader>()
+        let size = std::mem::size_of::<GlobalHeader>()
             + std::mem::size_of::<u16>() * NUM_TRANS_IN_UNIT * num_devices;
         unsafe {
             let header = data.as_mut_ptr() as *mut GlobalHeader;
-            (*header).ctrl_flag |= FPGAControlFlags::OUTPUT_ENABLE;
+            (*header).cpu_flag |= CPUControlFlags::WRITE_BODY;
 
-            let mut cursor = data.as_mut_ptr().add(std::mem::size_of::<GlobalHeader>());
-            let byte_size = NUM_TRANS_IN_UNIT * std::mem::size_of::<u16>();
+            let mut cursor = data.as_mut_ptr().add(std::mem::size_of::<GlobalHeader>()) as *mut u16;
             for i in 0..num_devices {
-                std::ptr::copy_nonoverlapping(
-                    gain.data()[i].as_ptr() as *const u8,
-                    cursor,
-                    byte_size,
-                );
-                cursor = cursor.add(byte_size);
+                std::ptr::copy_nonoverlapping(gain.data()[i].as_ptr(), cursor, NUM_TRANS_IN_UNIT);
+                cursor = cursor.add(NUM_TRANS_IN_UNIT);
             }
         }
+        size
     }
 
-    pub fn pack_seq(
-        seq: &mut PointSequence,
-        geometry: &Geometry,
-        data: &mut [u8],
-        size: &mut usize,
-    ) {
+    pub fn pack_seq(seq: &mut PointSequence, geometry: &Geometry, data: &mut [u8]) -> usize {
+        if seq.finished() {
+            return std::mem::size_of::<GlobalHeader>();
+        }
+
         let num_devices = geometry.num_devices();
 
-        *size = std::mem::size_of::<GlobalHeader>()
+        let size = std::mem::size_of::<GlobalHeader>()
             + std::mem::size_of::<u16>() * NUM_TRANS_IN_UNIT * num_devices;
 
         let header = data.as_mut_ptr() as *mut GlobalHeader;
         let mut offset = 1;
         unsafe {
-            let mut cursor = data.as_mut_ptr().add(std::mem::size_of::<GlobalHeader>()) as *mut u16;
+            (*header).cpu_flag |= CPUControlFlags::WRITE_BODY;
 
+            let mut cursor = data.as_mut_ptr().add(std::mem::size_of::<GlobalHeader>()) as *mut u16;
             if seq.sent() == 0 {
-                (*header).command_flag |= CPUControlFlags::SEQ_BEGIN;
+                (*header).cpu_flag |= CPUControlFlags::SEQ_BEGIN;
                 for i in 0..num_devices {
                     cursor
                         .add(i * NUM_TRANS_IN_UNIT + 1)
-                        .write(seq.sampling_freq_div());
+                        .write((*seq.sampling_freq_div() - 1) as u16);
                     cursor
                         .add(i * NUM_TRANS_IN_UNIT + 2)
                         .write((geometry.wavelength * 1000.0) as u16);
@@ -156,8 +159,8 @@ impl Logic {
             );
 
             if seq.sent() + send_size >= seq.size() {
-                (*header).command_flag |= CPUControlFlags::SEQ_END;
-                (*header).ctrl_flag |= FPGAControlFlags::OUTPUT_ENABLE;
+                (*header).cpu_flag |= CPUControlFlags::SEQ_END;
+                (*header).fpga_flag |= FPGAControlFlags::OUTPUT_ENABLE;
             }
 
             let fixed_num_unit = 256.0 / geometry.wavelength;
@@ -166,50 +169,50 @@ impl Logic {
                 let mut focus_cursor = cursor.add(offset) as *mut SeqFocus;
                 for i in 0..send_size {
                     let cp = seq.control_points()[seq.sent() + i];
-                    let v64 = geometry.local_position(device, cp.0);
-                    let x = v64[0] * fixed_num_unit;
-                    let y = v64[1] * fixed_num_unit;
-                    let z = v64[2] * fixed_num_unit;
-                    (*focus_cursor).set(x as i32, y as i32, z as i32, cp.1);
+                    let v64 = geometry.local_position(device, cp.0) * fixed_num_unit;
+                    (*focus_cursor).set(v64[0] as i32, v64[1] as i32, v64[2] as i32, cp.1);
                     focus_cursor = focus_cursor.add(1);
                 }
                 cursor = cursor.add(NUM_TRANS_IN_UNIT);
             }
             seq.send(send_size);
         }
+
+        size
     }
 
-    pub fn pack_gain_seq(
-        seq: &mut GainSequence,
-        geometry: &Geometry,
-        data: &mut [u8],
-        size: &mut usize,
-    ) {
+    pub fn pack_gain_seq(seq: &mut GainSequence, geometry: &Geometry, data: &mut [u8]) -> usize {
+        if seq.finished() {
+            return std::mem::size_of::<GlobalHeader>();
+        }
+
         let num_devices = geometry.num_devices();
 
-        *size = std::mem::size_of::<GlobalHeader>()
+        let size = std::mem::size_of::<GlobalHeader>()
             + std::mem::size_of::<u16>() * NUM_TRANS_IN_UNIT * num_devices;
 
         let header = data.as_mut_ptr() as *mut GlobalHeader;
         let seq_sent = *seq.gain_mode() as usize;
         unsafe {
+            (*header).cpu_flag |= CPUControlFlags::WRITE_BODY;
+
             if seq.sent() == 0 {
                 let cursor = data.as_mut_ptr().add(std::mem::size_of::<GlobalHeader>()) as *mut u16;
-                (*header).command_flag |= CPUControlFlags::SEQ_BEGIN;
+                (*header).cpu_flag |= CPUControlFlags::SEQ_BEGIN;
                 for i in 0..num_devices {
                     cursor.add(i * NUM_TRANS_IN_UNIT).write(seq_sent as _);
                     cursor
                         .add(i * NUM_TRANS_IN_UNIT + 1)
-                        .write(seq.sampling_freq_div());
+                        .write((*seq.sampling_freq_div() - 1) as u16);
                     cursor.add(i * NUM_TRANS_IN_UNIT + 2).write(seq.size() as _);
                 }
                 seq.send(1);
-                return;
+                return size;
             }
 
             if seq.sent() + seq_sent > seq.size() {
-                (*header).command_flag |= CPUControlFlags::SEQ_END;
-                (*header).ctrl_flag |= FPGAControlFlags::OUTPUT_ENABLE;
+                (*header).cpu_flag |= CPUControlFlags::SEQ_END;
+                (*header).fpga_flag |= FPGAControlFlags::OUTPUT_ENABLE;
             }
 
             let mut cursor = data.as_mut_ptr().add(std::mem::size_of::<GlobalHeader>()) as *mut u16;
@@ -268,22 +271,27 @@ impl Logic {
 
             seq.send(seq_sent);
         }
+
+        size
     }
 
     pub fn pack_delay_offset(
         delay_offsets: &[[u16; NUM_TRANS_IN_UNIT]],
         num_devices: usize,
         data: &mut [u8],
-        size: &mut usize,
-    ) {
-        *size = std::mem::size_of::<GlobalHeader>()
+    ) -> usize {
+        let size = std::mem::size_of::<GlobalHeader>()
             + std::mem::size_of::<u16>() * NUM_TRANS_IN_UNIT * num_devices;
+        let header = data.as_mut_ptr() as *mut GlobalHeader;
         unsafe {
+            (*header).cpu_flag |= CPUControlFlags::WRITE_BODY;
             let mut cursor = data.as_mut_ptr().add(std::mem::size_of::<GlobalHeader>()) as *mut u16;
             for delay in delay_offsets {
                 std::ptr::copy_nonoverlapping(delay.as_ptr(), cursor, NUM_TRANS_IN_UNIT);
                 cursor = cursor.add(NUM_TRANS_IN_UNIT);
             }
         }
+
+        size
     }
 }
