@@ -4,7 +4,7 @@
  * Created Date: 24/05/2021
  * Author: Shun Suzuki
  * -----
- * Last Modified: 18/06/2021
+ * Last Modified: 14/10/2021
  * Modified By: Shun Suzuki (suzuki@hapis.k.u-tokyo.ac.jp)
  * -----
  * Copyright (c) 2021 Hapis Lab. All rights reserved.
@@ -16,6 +16,7 @@ use anyhow::Result;
 /// Modulation contains the amplitude modulation data.
 pub trait Modulation: Send {
     fn build(&mut self) -> Result<()>;
+    fn rebuild(&mut self) -> Result<()>;
     fn buffer(&self) -> &[u8];
     fn finished(&self) -> bool {
         self.remaining() == 0
@@ -28,6 +29,6 @@ pub trait Modulation: Send {
     }
     fn sent(&self) -> usize;
     fn send(&mut self, sent: usize);
-    fn sampling_frequency_division(&self) -> u16;
+    fn sampling_frequency_division(&mut self) -> &mut usize;
     fn sampling_freq(&self) -> f64;
 }
